def plot_new_sample(df, x):

    X = df.iloc[:, 1:13]
    y = df["label"].replace({"Control": 0, "Covid": 1})  # .value_counts()
    x = x.values.reshape(1, -1)

    scaler = StandardScaler()
    scaler.fit(X.values)
    X = scaler.transform(X.values)

    means = np.array(
        [
            243.94969928,
            256.31424094,
            313.33404167,
            311.02738043,
            284.31009239,
            307.20553442,
            293.71868478,
            298.39285688,
            276.90539674,
            303.74205978,
            262.39199638,
            228.63297464,
        ]
    )
    vars = np.array(
        [
            32.38231873,
            15.39840351,
            0.30922943,
            0.11581567,
            14.18706441,
            0.67305964,
            2.09758948,
            0.96071839,
            7.81709395,
            1.53088058,
            4.84237249,
            13.75799395,
        ]
    )

    # means = scaler.mean_
    # vars = scaler.var_

    print("mean: ", means)
    print("var: ", vars)

    # X = scale_data(X.values, means=means,stds=vars **0.5)
    x = scale_data(x, means=means, stds=vars ** 0.5)

    print("scaled_new_data: ", x)

    pca = PCA(n_components=2)
    X = pca.fit_transform(X)

    # # print('Scaled X: ', X)

    print("before pca: ", x)
    x = pca.transform(x)
    print("after pca: ", x)

    # x = scaler.transform(x)
    # print(x)

    # pipe = Pipeline([
    #     ('scale', StandardScaler()),
    #     ('reduce_dims', PCA(n_components=2))])

    # pipe.fit(X.values)
    # X = pipe.transform(X.values)

    # print('pca_comp: ', x)

    group = y
    # group = y[:-10]
    cdict = {0: "blue", 1: "red"}
    class_label = {0: "Control", 1: "Covid"}

    scatter_x = X[:, 0]
    scatter_y = X[:, 1]

    fig, ax = plt.subplots()
    for g in np.unique(group):
        ix = np.where(group == g)
        ax.scatter(
            scatter_x[ix], scatter_y[ix], c=cdict[g], label=class_label[g], alpha=0.5
        )
    ax.scatter(x[:, 0], x[:, 1], c="black", label="new sample", s=70)
    ax.legend()
    ax.set_title("New data cluster assignment")
    ax.set_xlabel("PC1")
    ax.set_ylabel("PC2")
    # plt.show()
    fig.savefig("./static/img/sample_cluster.jpeg")



import pandas as pd
import os
import random
import numpy as np
import re
import warnings


SENSOR_FEATURES = [
    "sensor_1",
    "sensor_2",
    "sensor_3",
    "sensor_4",
    "sensor_5",
    "sensor_6",
    "sensor_7",
    "sensor_8",
    "sensor_9",
    "sensor_10",
    "sensor_11",
    "sensor_12",
    "sensor_13",
    "sensor_14",
    "sensor_15",
    "sensor_16",
    "sensor_17",
    "sensor_18",
    "sensor_19",
    "sensor_20",
    "sensor_21",
    "sensor_22",
    "sensor_23",
    "sensor_24",
]



def find_header_value(field, filename):
    """Finds the location of a keyword (exp_name, date, etc.) from the header of the .txt file

    Args:
        field (_type_): _description_
        filename (_type_): _description_

    Returns:
        _type_: _description_
    """

    with open(filename) as f:
        for line_no, line in enumerate(f):
            if line.startswith(field):
                # Get the part of the string after the field name
                end_of_string = line[len(field) :]
                string = end_of_string[:-1]
                # break
                return string, line_no


def get_exp_stage(
    time, data_points_per_sec=4, baseline=2, absorb=7, pause=1, desorb=5, flush=23
):

    # data_points_per_sec: data point per second

    baseline_time = baseline * data_points_per_sec
    if time <= baseline_time:
        return "baseline"
    absorb_time = baseline_time + absorb * data_points_per_sec
    if time <= absorb_time:
        return "absorb"
    pause_time = absorb_time + pause * data_points_per_sec
    if time <= pause_time:
        return "pause"
    desorb_time = pause_time + desorb * data_points_per_sec
    if time <= desorb_time:
        return "desorb"
    flush_time = desorb_time + flush * data_points_per_sec
    if time <= flush_time:
        return "flush"
    wait_time = flush_time + flush * data_points_per_sec
    if time <= wait_time:
        return "wait"
    else:
        warnings.warn("Warning...........Can't process stage")
        return "nan"


def rename_columns(df):

    for col in df.columns:
        if "Sen" in col:
            new_col = re.findall("\d+", col)[0]
            df.rename(columns={col: f"sensor_{new_col}"}, inplace=True)
    df.rename(columns={"Data Points": "timesteps", "Humidity (%r.h.)":"humidity"}, inplace=True)
    # df.columns = df.columns.str.replace(r"[( r.h. %)]", "")

    return df


def get_label(f):

    filename = f.split("/")[-1]
    exp_name, _ = find_header_value("Name of the experiment =", f)

    if (
        ("Neg" in filename)
        or ("Control" in filename)
        or ("CONTROL" in filename)
        or ("Neg" in exp_name)
        or ("Control" in exp_name)
        or ("CONTROL" in exp_name)
        or ("Neg" in filename)
    ):
        return "Control"
    elif (
        ("Pos" in filename)
        or ("Covid" in filename)
        or ("COVID" in filename)
        or ("Pos" in exp_name)
        or ("Covid" in exp_name)
        or ("COVID" in exp_name)
        or ("Pos" in filename)
    ):
        return "Covid"
    else:
        warnings.warn("Warning...........Can't get label")
        return "Other"


def get_data_points_per_sec(f):
    global data_points_per_sec
    field = "Acquired data point per second = "
    data_points_per_sec = float(find_header_value(field, f)[0])
    return data_points_per_sec


def get_exp_stage_duration(f):

    # global baseline, absorb, pause, desorb, flush

    baseline = float(find_header_value("Base", f)[0][-4:])
    absorb = float(find_header_value("Absorb = ", f)[0])
    pause = float(find_header_value("Pause = ", f)[0])
    desorb = float(find_header_value("Desorb = ", f)[0])
    flush = float(find_header_value("Flush = ", f)[0])

    return baseline, absorb, pause, desorb, flush


def offset_one_sample(df_temp, baseline, data_points_per_sec):

    datapoints = int(baseline * data_points_per_sec)
    avg_baseline = df_temp[SENSOR_FEATURES].iloc[:datapoints].mean().values

    df_temp[SENSOR_FEATURES] = df_temp[SENSOR_FEATURES].apply(
        lambda row: row - avg_baseline, axis=1
    )

    return df_temp


def offset_batch_samples(df, id_col="exp_unique_id", baseline=2, data_points_per_sec=4):
    """normalises all sensors in each cycle of experiment by subtracting the average of the sensor voltage in the 1st 2 sec from sensor voltage in other periods 

    Args:
        df (pandas dataframe): dataframe of cyclic sensor data
        sec (int, optional): number of seconds to use for offsetting. Defaults to 2.
        id_col (str, optional): unique id of each cycle of experiment. Defaults to 'exp_unique_id'.

    Returns:
        pandas dataframe: _description_
    """
    samples = df.groupby(id_col)
    df_list = []
    for i, sample in samples:
        df_list.append(offset_one_sample(sample, baseline=baseline, data_points_per_sec=data_points_per_sec))
    return pd.concat(df_list)



def gradient_one_sample(df):
    df[SENSOR_FEATURES] = df[SENSOR_FEATURES].diff().fillna(df[SENSOR_FEATURES].diff().shift(-1))
    return df


def gradient_batch_samples(df, id_col="exp_unique_id"):
    """normalises all sensors in each cycle of experiment by subtracting the average of the sensor voltage in the 1st 2 sec from sensor voltage in other periods 

    Args:
        df (pandas dataframe): dataframe of cyclic sensor data
        sec (int, optional): number of seconds to use for offsetting. Defaults to 2.
        id_col (str, optional): unique id of each cycle of experiment. Defaults to 'exp_unique_id'.

    Returns:
        pandas dataframe: _description_
    """
    samples = df.groupby(id_col)
    df_list = []
    for i, sample in samples:
        df_list.append(gradient_one_sample(sample))
    return pd.concat(df_list)



def fillna_columns(df_temp):

    num = df_temp._get_numeric_data()
    num[num < 0] = np.nan
    num = num.fillna(num.mean())
    return df_temp


def preprocess_single_file(
    f,
    parse_time=True,
    parse_filename=True,
    rename_column=True,
    fillna=True,
    offset=False,
    has_label=True,
):
    """preprocess a single cycle (one txt file) of experiment

    Args:
        f (str): file path with txt extension
        rename_column (bool, optional): rename columns. Defaults to True.
        fillna (bool, optional): replaces extreme negative values with mean of column. Defaults to True.
        offset (bool, optional): normalises all sensors in each cycle of experiment by subtracting the average of the sensor voltage in the 1st 2 sec from sensor voltage in other periods . Defaults to True.
        has_label (bool, optional): extracts label and add to dataframe. Defaults to True.

    Returns:
        pandas dataframe: dataframe of features in a cycle(one txt file)
    """

    _, line_no = find_header_value("Data Points", f)

    df_temp = pd.read_csv(f, sep="\t", header=(line_no - 1))

    data_points_per_sec = get_data_points_per_sec(f)
    baseline, absorb, pause, desorb, flush = get_exp_stage_duration(f)

    df_temp['exp_unique_id'] = 0

    df_temp["measurement_stage"] = df_temp["Data Points"].apply(
        get_exp_stage,
        data_points_per_sec=data_points_per_sec,
        baseline=baseline,
        absorb=absorb,
        pause=pause,
        desorb=desorb,
        flush=flush,
    )

    if parse_time:
        date, _ = find_header_value("Date = ", f)
        df_temp["date_exp"], _ = find_header_value("Date = ", f)
        df_temp["time_elapsed"] = df_temp.index / 4
        time_start, _ = find_header_value("Time = ", f)
        time_elapsed = df_temp.index / 4
        df_temp["datetime_exp_start"] = time_start
        df_temp["datetime_exp"] = pd.to_datetime(
            date + " " + time_start
        ) + pd.to_timedelta(time_elapsed, unit="s")



    if parse_filename:
        df_temp["filename"] = f.split("/")[-1]



    df_temp["exp_name"] = find_header_value("Name of the experiment = ", f)[0][1:-1]

    if rename_column:
        df_temp = rename_columns(df_temp)

    if fillna:
        df_temp = fillna_columns(df_temp)

    if offset:
        df_temp = offset_batch_samples(df_temp, baseline=baseline, data_points_per_sec=data_points_per_sec)


    repeat_no = find_header_value("Repeat No. = ", f)[0].split("/")[0]
    df_temp["repeat_no"] = repeat_no

    if has_label:
        df_temp["result"] = get_label(f)
        df_temp = reorder_columns(df_temp)
        return df_temp

    df_temp = reorder_columns(df_temp)

    return df_temp


def get_all_files(parent_dir):
    all_files = []
    for path, subdirs, files in os.walk(parent_dir):
        for name in files:
            if (name.endswith("txt")) and (len(name) > 25) and ("Wash" not in name):
                all_files.append(os.path.join(path, name))
    return all_files


def get_single_repeat(df):
    """select one (chosen at random) of repeat experiemnts

    Args:
        df (pandas dataframe): dataframe of cyclic sensor data with repeat experiments

    Returns:
        pandas dataframe: dataframe with only one of each repeat experiemnt
    """

    g = df.groupby(["exp_name"])

    exp_names = list(set(df["exp_name"]))
    all = []
    for i in exp_names:
        df = g.get_group(i)

        uniq_files = list(df["filename"].unique())
        file_chosen = random.choice(uniq_files)
        df = df[df["filename"] == file_chosen]
        all.append(df)
    return pd.concat(all)


def reorder_columns(df):
    """reorders features with exp_unique_id as first column asnd result as last

    Args:
        df (pandas dataframe): _description_

    Returns:
        pandas dataframe: _description_
    """
    feature_len = len(df.columns)

    try:
        df.insert(feature_len-1, 'result', df.pop('result'))
        df.insert(0, 'exp_unique_id', df.pop('exp_unique_id'))
    except:
        pass



    # col_list = df.columns.tolist()
    # new_col_list = [col_list[-1]] + [col_list[-2]] + col_list[:-2]
    # df = df[new_col_list]

    return df


def preprocess_all_files(
    path_to_measurements,
    parse_time=True,
    parse_filename=True,
    rename_column=True,
    no_repeat=False,
    fillna=True,
    offset=False,
    has_label=True,
):
    """preprocess all cycle (all txt file) of experiment

    Args:
        path_to_measurements (str): folder path to measurements
        parse_time (bool, optional): extract time features. Defaults to True.
        parse_filename (bool, optional): adds filename to dataframe. Defaults to True.
        rename_column (bool, optional): rename columns. Defaults to True.
        no_repeat (bool, optional): _description_. Defaults to False.
        fillna (bool, optional): replaces extreme negative values with mean of column. Defaults to True.
        offset (bool, optional): normalises all sensors in each cycle of experiment by subtracting the average of the sensor voltage in the 1st 2 sec from sensor voltage in other periods . Defaults to True.
        has_label (bool, optional): extracts label and add to dataframe. Defaults to True.

    Returns:
        pandas dataframe: dataframe of features in a cycle(one txt file)
    """

    list_files = get_all_files(path_to_measurements)

    df_list = []
    for i, f in enumerate(list_files):

        df_temp = preprocess_single_file(
            f,
            parse_time=parse_time,
            parse_filename=parse_filename,
            offset=offset,
            fillna=fillna,
            has_label=has_label,
        )
        df_temp["exp_unique_id"] = i
        df_list.append(df_temp)

    df = pd.concat(df_list)
    df = reorder_columns(df)


    if no_repeat:
        return get_single_repeat(df)
    else:
        return df



import numpy as np
import pandas as pd
import xgboost as xgb
from boruta import BorutaPy
from tsfresh.utilities.dataframe_functions import impute
from tsfresh.feature_extraction import (
    ComprehensiveFCParameters,
    extract_features,
)
from tsfresh import extract_features
from .preprocess import offset_batch_samples, gradient_batch_samples


FEATURES = [
    "exp_unique_id",
    "timesteps",
    "sensor_1",
    "sensor_2",
    "sensor_3",
    "sensor_4",
    "sensor_5",
    "sensor_6",
    "sensor_7",
    "sensor_8",
    "sensor_9",
    "sensor_10",
    "sensor_11",
    "sensor_12",
    "sensor_13",
    "sensor_14",
    "sensor_15",
    "sensor_16",
    "sensor_17",
    "sensor_18",
    "sensor_19",
    "sensor_20",
    "sensor_21",
    "sensor_22",
    "sensor_23",
    "sensor_24",
]

TARGET_COL = "result"

extraction_settings = ComprehensiveFCParameters()


def get_all_features(
    X,
    unique_id="exp_unique_id",
    # label="result",
    timesteps="timesteps",
    features=FEATURES,
):

    features = [col for col in X if col.startswith('sensor')]
    features = [unique_id, timesteps] + features

    X = X[features]

    # X = df.drop(label, axis=1)
    # y = df.groupby(unique_id).first()[label]

    X_extracted = extract_features(
        X,
        column_id=unique_id,
        column_sort=timesteps,
        default_fc_parameters=extraction_settings,
        # we impute = remove all NaN features automatically
        impute_function=impute,
    )

    return X_extracted


xgb = xgb.XGBClassifier()



def select_relevant_features(X, y, tree_model=xgb):

    boruta = BorutaPy(
            estimator=tree_model,
            n_estimators="auto",
            max_iter=50,  # number of trials to perform
        )

    # fit Boruta (it accepts np.array, not pd.DataFrame)
    boruta.fit(np.array(X), np.array(y))

    # green and blue area are the important features identified by boruta algorithm
    green_area = X.columns[boruta.support_].to_list()
    blue_area = X.columns[boruta.support_weak_].to_list()

    relevant_features = green_area + blue_area
    X = X[relevant_features]

    return  X

unique_id="exp_unique_id"
label="result"
timesteps="timesteps"

def add_offset_gradient(X_raw, offset=False, gradient=False):

    
    X_all = []
    X_all.append(X_raw)

    if offset:
        X_offset = offset_batch_samples(X_raw)
        X_offset = X_offset.add_suffix(f'_offset')
        X_all.append(X_offset)

    if gradient:
        X_gradient = gradient_batch_samples(X_raw)
        X_gradient = X_gradient.add_suffix(f'_gradient')
        X_all.append(X_gradient)

    

    X_all = pd.concat(X_all, axis=1)

    return X_all

def get_training_features(df, offset=False, gradient=False, tree_model=xgb):
    y = df.groupby(unique_id).first()[label]
    X = df.drop(label, axis=1)
    X = add_offset_gradient(X, offset, gradient)
    print('Extracting all features')
    X = get_all_features(X)
    print('Selecting relevant features')
    X = select_relevant_features(X, y, tree_model=tree_model)

    df = X.join(y)

    return df



















































# import numpy as np
# import pandas as pd
# import xgboost as xgb
# from boruta import BorutaPy
# from tsfresh.utilities.dataframe_functions import impute
# from tsfresh.feature_extraction import (
#     ComprehensiveFCParameters,
#     extract_features,
# )
# from tsfresh import extract_features
# from .preprocess import offset_batch_samples, gradient_batch_samples


# FEATURES = [
#     "exp_unique_id",
#     "timesteps",
#     "sensor_1",
#     "sensor_2",
#     "sensor_3",
#     "sensor_4",
#     "sensor_5",
#     "sensor_6",
#     "sensor_7",
#     "sensor_8",
#     "sensor_9",
#     "sensor_10",
#     "sensor_11",
#     "sensor_12",
#     "sensor_13",
#     "sensor_14",
#     "sensor_15",
#     "sensor_16",
#     "sensor_17",
#     "sensor_18",
#     "sensor_19",
#     "sensor_20",
#     "sensor_21",
#     "sensor_22",
#     "sensor_23",
#     "sensor_24",
#     "result",
# ]

# TARGET_COL = "result"

# extraction_settings = ComprehensiveFCParameters()





# def get_features(
#     df,
#     unique_id="exp_unique_id",
#     label="result",
#     timesteps="timesteps",
#     features=FEATURES,
#     offset=False,
#     gradient=False
# ):

#     df = df[features]

#     X = df.drop(label, axis=1)
#     y = df.groupby(unique_id).first()[label]

#     X_all = []



#     print('Extracting feature from raw signals')

#     X_raw = extract_features(
#         X,
#         column_id=unique_id,
#         column_sort=timesteps,
#         default_fc_parameters=extraction_settings,

#         # we impute = remove all NaN features automatically
#         impute_function=impute,
#     )

#     X_all.append(X_raw)



#     if offset:
#         print('Extracting features from offset signals')
#         X_offset = offset_batch_samples(X)

#         X_offset = extract_features(
#             X,
#             column_id=unique_id,
#             column_sort=timesteps,
#             default_fc_parameters=extraction_settings,

#             # we impute = remove all NaN features automatically
#             impute_function=impute,
#         )

#         X_all.append(X_offset)

#     if gradient:
#         print('Extracting features from gradient signals')
#         X_gradient = gradient_batch_samples(X)
#         X_gradient = extract_features(
#             X,
#             column_id=unique_id,
#             column_sort=timesteps,
#             default_fc_parameters=extraction_settings,

#             # we impute = remove all NaN features automatically
#             impute_function=impute,
#         )
#         X_all.append(X_gradient) 

    

#     X = pd.concat(X_all, axis=1)


#     df = X.join(y)

#     return df, X, y


# xgb = xgb.XGBClassifier()


# def get_relevant_features(X, y, tree_model=xgb):

#     boruta = BorutaPy(
#         estimator=tree_model,
#         n_estimators="auto",
#         max_iter=100,  # number of trials to perform
#     )

#     # fit Boruta (it accepts np.array, not pd.DataFrame)
#     boruta.fit(np.array(X), np.array(y))

#     # green and blue area are the important features identified by boruta algorithm
#     green_area = X.columns[boruta.support_].to_list()
#     blue_area = X.columns[boruta.support_weak_].to_list()

#     relevant_features = green_area + blue_area
#     X = X[relevant_features]

#     df = X.join(y)

#     return df, X, y



import joblib
from io import BytesIO
import boto3
import mlflow
from mlflow.tracking import MlflowClient
from tsfresh.feature_extraction import settings, extract_features
from .feature_extraction import add_offset_gradient



REMOTE_TRACKING_URI = 'http://ec2-3-10-175-206.eu-west-2.compute.amazonaws.com:5000/'


def load_model(path):
    """ 
    Function to load a joblib file from an s3 bucket or local directory.
    Arguments:
    * path: an s3 bucket or local directory path where the file is stored
    Outputs:
    * file: Joblib file loaded
    """

    # Path is an s3 bucket
    if path[:5] == "s3://":
        s3_bucket, s3_key = path.split("/")[2], path.split("/")[3:]
        s3_key = "/".join(s3_key)
        with BytesIO() as f:
            boto3.client("s3").download_fileobj(Bucket=s3_bucket, Key=s3_key, Fileobj=f)
            f.seek(0)
            file = joblib.load(f)

    # Path is a local directory
    else:
        with open(path, "rb") as f:
            file = joblib.load(f)

    return file



def get_model_and_features(model_uri, tracking_uri=REMOTE_TRACKING_URI):
    
    client = MlflowClient(tracking_uri=tracking_uri)
    
    # Load model as a Sklearn Model.
    loaded_model = mlflow.sklearn.load_model(model_uri)
    run_id = loaded_model.metadata.run_id
    relevant_features = client.get_run(run_id).data.tags['relevant_features'].replace("'", '')[1:-1].split(', ')
    offset = bool(client.get_run(run_id).data.tags['offset'])
    gradient = bool(client.get_run(run_id).data.tags['gradient'])

    return loaded_model, relevant_features, offset, gradient




def get_prediction_features(
    X, model_uri, tracking_uri=REMOTE_TRACKING_URI,  id="exp_unique_id", timesteps="timesteps"
):


    
    _, relevant_features, offset, gradient = get_model_and_features(model_uri)

    X = add_offset_gradient(X, offset=offset, gradient=gradient)
    # df = X.join(y)

    fc_parameters = settings.from_columns(relevant_features)

    prediction_data = extract_features(
        X, kind_to_fc_parameters=fc_parameters, column_id=id, column_sort=timesteps
    )

    prediction_data.columns = prediction_data.columns.str.replace('["]', "")


    return prediction_data



def make_prediction(df, model_uri, tracking_uri=REMOTE_TRACKING_URI,  id="exp_unique_id", timesteps="timesteps"):

    loaded_model, _, _, _ = get_model_and_features(model_uri, tracking_uri=REMOTE_TRACKING_URI)

    prediction_data = get_prediction_features(df, model_uri, tracking_uri,  id, timesteps)

    prediction = loaded_model.predict(prediction_data)

    return prediction




# !pip install raptor_functions pandas_profiling
import matplotlib.pyplot as plt
import pandas as pd
import numpy as np
from raptor_functions.datasets import get_data




# import ssl
# ssl._create_default_https_context = ssl._create_unverified_context
#
df = get_data()
#
from pandas_profiling import ProfileReport
profile = ProfileReport(df, title="Pandas Profiling Report", explorative=True)
profile.to_file("pandas_profiling_report.html")



import sweetviz as sv
my_report = sv.analyze(df)
my_report.show_html() # Default arguments will generate to "SWEETVIZ_REPORT.html"



from autoviz.AutoViz_Class import AutoViz_Class
AV = AutoViz_Class()

import nltk
nltk.download('wordnet')

_ = AV.AutoViz('df.csv')




import dtale
import dtale.app as dtale_app
# dtale_app.USE_COLAB = True
dtale_app.USE_NGROK = True
dtale.show(df)


# Importing the Packages:
import optuna
import mlflow
import os

# try:
#     from sklearn.utils import safe_indexing
# except ImportError:
#     from sklearn.utils import _safe_indexing

from pathlib import Path
from pycaret.classification import *
from sklearn.metrics import recall_score
from mlflow.models.signature import infer_signature

SENSORS_FEATURES = [
    "sensor_1",
    "sensor_2",
    "sensor_3",
    "sensor_4",
    "sensor_5",
    "sensor_6",
    "sensor_7",
    "sensor_8",
    "sensor_9",
    "sensor_10",
    "sensor_11",
    "sensor_12",
    "sensor_13",
    "sensor_14",
    "sensor_15",
    "sensor_16",
    "sensor_17",
    "sensor_18",
    "sensor_19",
    "sensor_20",
    "sensor_21",
    "sensor_22",
    "sensor_23",
    "sensor_24",
]

STAGES = ["baseline", "absorb", "pause", "desorb", "flush"]
TARGET_COL = "result"
REMOTE_TRACKING_URI = "http://ec2-3-10-175-206.eu-west-2.compute.amazonaws.com:5000/"


# 'http://ec2-3-10-210-150.eu-west-2.compute.amazonaws.com:5000/'


def get_plots(model):

    model_attributes = dir(model)

    plot_dir = os.path.join(os.getcwd(), "plots")
    Path(plot_dir).mkdir(parents=True, exist_ok=True)

    plot_model(model, plot="confusion_matrix", save=plot_dir)
    cm = os.path.join(plot_dir, "Confusion Matrix.png")
    # mlflow.log_artifact(cm)
    # log_artifact(cm)

    plot_model(model, plot="class_report", save=plot_dir)
    cr = os.path.join(plot_dir, "Class Report.png")
    # mlflow.log_artifact(cr)
    # log_artifact(cr)

    if "predict_proba" in model_attributes:
        try:
            plot_model(model, plot="auc", save=plot_dir)
            auc = os.path.join(plot_dir, "AUC.png")
            # mlflow.log_artifact(auc)

            plot_model(model, plot="pr", save=plot_dir)
            pr = os.path.join(plot_dir, "Precision Recall.png")
            # mlflow.log_artifact(pr)

            plot_model(model, plot="calibration", save=plot_dir)
            cc = os.path.join(plot_dir, "Calibration Curve.png")
            # mlflow.log_artifact(cc)
        except:
            pass

    if "feature_importances_" in model_attributes:
        plot_model(model, plot="feature", save=plot_dir)
        fi = os.path.join(plot_dir, "Feature Importance.png")
        # mlflow.log_artifact(fi)

    mlflow.log_artifact(plot_dir)


def specificity(actual, pred, pos_label=0):
    return recall_score(actual, pred, pos_label=pos_label)


def objective(trial, df, study_name, model_mode):



    mlflow.set_experiment(study_name)

    mlflow.set_tracking_uri(REMOTE_TRACKING_URI)


    df_offset = df.filter(regex='offset')
    df_gradient = df.filter(regex='gradient')

    if len(df_offset.columns) > 0:
        offset = True
    else:
        offset = False

    if len(df_gradient.columns) > 0:
        gradient = True
    else:
        gradient = False
    

    df_copy = df.copy(deep=True)

    relevant_features = list(df_copy.drop(TARGET_COL, axis=1).columns)

    mlflow.start_run()

    # remove unwanted characters from the column names
    df_copy.columns = df_copy.columns.str.replace('["]', "")

    features = df_copy.drop(TARGET_COL, axis=1)
    df_copy["result"] = df_copy["result"].replace({"Control": 0, "Covid": 1})

    clf = setup(
        df_copy,
        target=TARGET_COL,
        silent=True,
        log_plots=True,
        experiment_name=study_name,
        preprocess=False,
        remove_multicollinearity=True,
        multicollinearity_threshold=1.0
    )

    add_metric("specificity", "Specificity", specificity)
    remove_metric("MCC")
    remove_metric("Kappa")

    MODELS = dict(models()["Name"])

    if model_mode == "random":

        model_id = trial.suggest_categorical("model", list(MODELS.keys()))
        model_name = MODELS[model_id]
        model = create_model(model_id)
        predict_model(model)
        metrics = pull().iloc[:, 1:]
        keys = metrics.columns
        values = metrics.values[0]
        metrics = dict(zip(keys, values))

        get_plots(model)

        signature = infer_signature(features, predict_model(model, data=features))
        mlflow.sklearn.log_model(model, artifact_path=model_name, signature=signature)

    # elif model_mode == 'xgb':

    #     model = create_model('xgb')
    #     predict_model(model)
    #     metrics = pull().iloc[:,1:]
    #     keys = metrics.columns
    #     values = metrics.values[0]
    #     metrics = dict(zip(keys, values))

    #     get_plots(model)
    #     mlflow.sklearn.log_model(model, artifact_path=model_name)

    else:

        model = compare_models()

        model_name = model.__class__.__name__
        predict_model(model)
        metrics = pull().iloc[:, 1:]
        keys = metrics.columns
        values = metrics.values[0]
        metrics = dict(zip(keys, values))

        get_plots(model)

        signature = infer_signature(features, predict_model(model, data=features))
        mlflow.sklearn.log_model(model, artifact_path=model_name, signature=signature)

    model_params = model.get_params()

    mlflow.log_metrics(metrics)

    try:
        # mlflow.set_tag('features', features)
        mlflow.set_tag("relevant_features", relevant_features)

    except:
        pass

    mlflow.set_tag("model_name", model_name)
    mlflow.set_tag("offset", offset)
    mlflow.set_tag("gradient", gradient)
    mlflow.set_tag("model_params", model_params)


    mlflow.set_tag("model_mode", model_mode)

    mlflow.end_run()


def train_experiments(
    df, study_name="raptor", direction="maximize", model_mode="random", n_trials=5
):
    """trains several models during different trials and logs them. Experiemnt can be tracked on "http://ec2-3-10-175-206.eu-west-2.compute.amazonaws.com:5000/"


    Args:
        df (pandas dataframe): dataframe of cyclic sensor data to be used for training
        study_name (str, optional): optuna study name to use. Defaults to 'raptor'.
        direction (str, optional): direction of objective. Defaults to 'maximize'.
        model_mode (str, optional): _description_. Defaults to 'random'.
        n_trials (int, optional): number of times to run experiments. Defaults to 5.
    """

    study = optuna.create_study(study_name=study_name, direction=direction)
    study.optimize(
        lambda trial: objective(trial, df, study_name, model_mode), n_trials=n_trials
    )

    print("Click on this link to track experiments: ", REMOTE_TRACKING_URI)



import os
import sys
import glob
import numpy as np
import pandas as pd


PACKAGE_FOLDER =  os.environ.get('PACKAGE_FOLDER')
<<<<<<< HEAD
PACKAGE_FOLDER = '/media/psf/Desktop/Projects/Raptor/raptor_functions'
PACKAGE_FOLDER = '/raptor/raptor_functions' #'/home/raptor_functions' '/raptor/raptor_functions
=======
PACKAGE_FOLDER = '/Users/danielfiuzadosil/Documents/GitHub/raptor_functions/raptor_functions'

>>>>>>> e114c4e285d423eaf9202881d7c7be055031a323
print(PACKAGE_FOLDER)
# sys.path.insert(0, PACKAGE_FOLDER)
sys.path.append(PACKAGE_FOLDER)


import warnings
warnings.filterwarnings('ignore')

import pandas as pd
import mlflow
# import xgboost as xgb
from sklearn.ensemble import RandomForestClassifier
from raptor_functions.supervised.train import train_experiments
from raptor_functions.supervised.datasets import get_data
from raptor_functions.supervised.feature_extraction import get_training_features



df = get_data('handheld_data')

forest = RandomForestClassifier()
df = get_training_features(df, offset=False, gradient=False, tree_model=forest)

train_experiments(df)